summary refs log tree commit diff
diff options
context:
space:
mode:
authorKegan Dougal <kegan@matrix.org>2015-02-04 17:39:51 +0000
committerKegan Dougal <kegan@matrix.org>2015-02-04 17:39:51 +0000
commit543e84fe70bdc0af6be5feb237de2d18adf352ab (patch)
tree37a8ff52152ae413d5a6c00bb2bae4f825620646
parentUpdate user/alias query APIs to use new format of SimpleHttpClient.get_json (diff)
downloadsynapse-543e84fe70bdc0af6be5feb237de2d18adf352ab.tar.xz
Add SimpleHttpClient.put_json with the same semantics as get_json.
-rw-r--r--synapse/http/client.py48
1 files changed, 46 insertions, 2 deletions
diff --git a/synapse/http/client.py b/synapse/http/client.py
index fee8c901a2..575510637e 100644
--- a/synapse/http/client.py
+++ b/synapse/http/client.py
@@ -83,7 +83,7 @@ class SimpleHttpClient(object):
 
     @defer.inlineCallbacks
     def get_json(self, uri, args={}):
         """ Gets some json from the given host and path
 
         Args:
             uri (str): The URI to request, not including query parameters
@@ -92,7 +92,8 @@ class SimpleHttpClient(object):
                 **Note**: The value of each key is assumed to be an iterable
                 and *not* a string.
         Returns:
-            Deferred: Succeeds when we get *any* 2xx HTTP response.
+            Deferred: Succeeds when we get *any* 2xx HTTP response, with the
+            HTTP body as JSON.
         Raises:
             On a non-2xx HTTP response.
         """
@@ -118,6 +119,49 @@ class SimpleHttpClient(object):
             # load it into JSON if they want.
             raise CodeMessageException(response.code, body)
 
+    @defer.inlineCallbacks
+    def put_json(self, uri, json_body, args={}):
+        """ Puts some json to the given URI.
+
+        Args:
+            uri (str): The URI to request, not including query parameters
+            json_body (dict): The JSON to put in the HTTP body,
+            args (dict): A dictionary used to create query strings, defaults to
+                None.
+                **Note**: The value of each key is assumed to be an iterable
+                and *not* a string.
+        Returns:
+            Deferred: Succeeds when we get *any* 2xx HTTP response, with the
+            HTTP body as JSON.
+        Raises:
+            On a non-2xx HTTP response.
+        """
+        if len(args):
+            query_bytes = urllib.urlencode(args, True)
+            uri = "%s?%s" % (uri, query_bytes)
+
+        json_str = json.dumps(json_body)
+
+        response = yield self.agent.request(
+            "PUT",
+            uri.encode("ascii"),
+            headers=Headers({
+                b"User-Agent": [AGENT_NAME],
+                "Content-Type": ["application/json"]
+            }),
+            bodyProducer=FileBodyProducer(StringIO(json_str))
+        )
+
+        body = yield readBody(response)
+
+        if 200 <= response.code < 300:
+            defer.returnValue(json.loads(body))
+        else:
+            # NB: This is explicitly not json.loads(body)'d because the contract
+            # of CodeMessageException is a *string* message. Callers can always
+            # load it into JSON if they want.
+            raise CodeMessageException(response.code, body)
+
 
 class CaptchaServerHttpClient(SimpleHttpClient):
     """
@@ -83,7 +83,7 @@ class SimpleHttpClient(object):
     @defer.inlineCallbacks
     def get_json(self, uri, args={}):
-        """ Gets some json from the given host and path
+        """ Gets some json from the given URI.
 
         Args: